import javax.naming.*;
import javax.naming.directory.*;
import java.io.*;
import java.util.*;

public class utils {

  //Loads properties from a file
  static public Properties getProps(String fn) {
    Properties properties = new Properties();
    File propsFile = new File(fn);
    try {
      FileInputStream in =
	new FileInputStream(propsFile);
       properties.load(new BufferedInputStream(in));
      in.close();
    } catch (Exception e) {
      System.out.println("getProps: "+e.getMessage());
      System.exit(1);
    }	
    return properties;
  }	

  static public boolean followReferral(ReferralException e) {
    byte[] reply = new byte[2];
    
    System.out.println("Referred to:");
    System.out.println((String)e.getReferralInfo());
    System.out.print("Follow? [y/n] ");
    try {
      System.in.read(reply);
    } catch (IOException ex) {
      System.out.println(ex.getMessage());
      return false;
    }
    return (reply[0] != 'n');
  }




  static void printSearchEnums(NamingEnumeration results) 
    throws NamingException {
    if (results == null) {
      System.out.println("Search Results are empty.");
    } else {
      while (results.hasMoreElements()) {
	SearchResult si =(SearchResult) results.next();
	System.out.println(si.getName());
	Attributes attrs = si.getAttributes();
	if (attrs == null) {
	  System.out.println("No attributes");
	} else {
	  for (NamingEnumeration ae = attrs.getAll();
	       ae.hasMoreElements();) {
	    Attribute attr =(Attribute) ae.next();
	    String id = attr.getID();
	    for (Enumeration vals = attr.getAll();
		 vals.hasMoreElements();
		 System.out.println("   "+id + ": " + vals.nextElement()));
	  }
	}
      }	
    }
  }

  static void printBindingList(NamingEnumeration be) {
    if (be == null) 
      System.out.println("Name bind list empty.");
    else {
      try {
	while (be.hasMore()) {
	  Binding b =(Binding) be.next();
	  System.out.println(b.getName()+": "+ b.getObject());
	}
      } catch (NamingException e) {
	System.out.println("printBindingList Failed: "+e.getMessage());
	System.exit(1);
      }
    }
  }


  static void printNameList(NamingEnumeration ne) {
    if (ne == null) 
      System.out.println("Name list empty.");
    else {
      try {
	while (ne.hasMore()) {
	  NameClassPair ncp=(NameClassPair)ne.next();
	  System.out.println(ncp.getName()+": "+ncp.getClassName());
	}
      } catch (NamingException e) {
	System.out.println("printNameList Failed: "+e.getMessage());
	System.exit(1);
      }
    }
  }

  static void printAttrSet(Attributes as) {
    if ((as == null) ||
      (as.size() == 0)){
      System.out.println("Attributes are empty or null.");
    } else {
      try {
	NamingEnumeration enum = as.getAll();
	while(enum.hasMoreElements()) {
	  Attribute attr = (Attribute)enum.next();
	  String type=attr.getID();
	  System.out.println(type+":");
	  Enumeration valEnums=attr.getAll();
	  while(valEnums.hasMoreElements())
	    System.out.println("   "+valEnums.nextElement());
	}
      } catch (NamingException e) {
	System.out.println("printAttrSet Failed:"+e.getMessage());
	System.exit(1);
      }
    }
  }

  //Returns attribute set for and add
  static public Attributes getAttrSet(Properties p) {
    Attributes as=new Attributes();
    Attribute at=null;
    try {
      int k=Integer.parseInt((String)p.get("example.add.attr.count"));
      String line=p.getProperty("example.add.objectclasses");
      Attribute oc=new Attribute("objectclass");
      StringTokenizer tok=new StringTokenizer(line," ");
      while (tok.hasMoreTokens()) 
	oc.add(tok.nextToken());
      as.put(oc);
      for(int i=0;i<k;i++) {
	String n=p.getProperty("example.add.name."+i);
	at = new Attribute(n);
	String v=p.getProperty("example.add.value."+i);
	at.add(v);
	as.put(at);
      }
    } catch (NumberFormatException nfe) {
      System.out.println("getAttrSet Failed: "+nfe.getMessage());
      System.exit(1);
    }
    return as;
  }

  static ModificationItem[] getMods(Properties p) {
    Attribute at=null;
    ModificationItem[] m=null;
    try {
      int k=Integer.parseInt((String)p.get("example.mod.attr.count"));
      m=new ModificationItem[k];
      for(int i=0;i<k;i++) {
	String n=p.getProperty("example.mod.attr.name."+i);
	at = new Attribute(n);
	String v=p.getProperty("example.mod.attr.value."+i);
	StringTokenizer tok=new StringTokenizer(v," ");
	while (tok.hasMoreTokens()) 
	  at.add(tok.nextToken());
	int op=
	  Integer.parseInt(p.getProperty("example.mod.attr.op."+i));
	m[i]=new ModificationItem(op,at);
      }
    } catch (NumberFormatException nfe) {
      System.out.println("getMods Failed: "+nfe.getMessage());
      System.exit(1);
    }
    return m;
  }


}
